/****************************************************************************
 *
 * Project: -----
 *
 * Copyright by Olimex Ltd. All rights reserved.
 *
 * File: gps.c
 * Description: Hardware-agnostic GPS module
 * Developer: Dimitar Dimitrov ( dinuxbg,gmail.com )
 *
 * Last change: $Date: 2009-03-25 13:48:21 +0200 (Wed, 25 Mar 2009) $
 * Revision: $Revision: 29 $
 * Id: $Id: gps.c 658 2009-03-25 11:48:21Z dimitar $
 *
 ****************************************************************************/

#include <assert.h>
#include <stddef.h>
#include <stdint.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <float.h>

#include "utils.h"

#include "gps.h"

#include "gps-hw.h"


/**
 * @file
 * Hardware-agnostic GPS module
 *
 * Very simple GPS parser.
 *
 * @note If message format and checksum are OK we assume that it is safe
 * enough to parse the integer values with atoi.
 */

/**
 * This allows us to represent floating point numbers with 5 integer
 * and 4 fractional _decimal_ digits, using fixed point 32-bit number.
 */
#define GPS_REAL_FIXEDPOINT	14

const char *gps_delim = ",";

static struct gps_gpgga gpgga;
static bool gpgga_valid;
static int gps_gpgga_error_counter;

static struct gps_gpgsa gpgsa;
static bool gpgsa_valid;
static int gps_gpgsa_error_counter;

static struct gps_gprmc gprmc;
static bool gprmc_valid;
static int gps_gprmc_error_counter;


static int gps_unrecognized_nmea_counter;

int gps_real_int(gps_real_t r)
{
	return r >> GPS_REAL_FIXEDPOINT;
}

unsigned int gps_real_fract10000(gps_real_t r)
{
	unsigned int fract;
	
	fract = (r % (1u << GPS_REAL_FIXEDPOINT)) * 10000;
	fract += (1u << GPS_REAL_FIXEDPOINT) / 2;	/* round to nearest */
	fract /= (1u << GPS_REAL_FIXEDPOINT);
	return fract;
}


unsigned int gps_real_fract100(gps_real_t r)
{
	return (gps_real_fract10000(r) + 50) / 100;
}


gps_real_t gps_real_create(int n, int fract, int fract_div)
{
	assert(n < (0xfffffffful >> GPS_REAL_FIXEDPOINT));

	fract = (fract * (1<<GPS_REAL_FIXEDPOINT)) / fract_div;
	assert(fract < (1<<GPS_REAL_FIXEDPOINT));

	return (n * (1<<GPS_REAL_FIXEDPOINT)) + fract;
}


static double gps_pos_real2double(gps_real_t pos)
{
	int deg, n;
	double res, min, tmp;

	n = gps_real_int(pos);
	deg = n/100;
	min = n%100;

	res = (double)deg;	/* whole part */

	/* fractional part of minutes */
	tmp = pos % (1u << GPS_REAL_FIXEDPOINT);
	tmp /= (1u << GPS_REAL_FIXEDPOINT);
	min += tmp;

	/* add normalized fractional part of degree */
	assert(min < 60.0);
	res += min / 60.0;

	/* convert to radians */
	res = res * M_PI / 180.0;

	return res;
}


void gps_norm_pos(const struct gps_position *pos, double *lat, double *lon)
{
	double f;

	/* convert latitude */
	f = gps_pos_real2double(pos->latitude);
	if(pos->ew == 'S') {
		f *= -1;
	}
	*lat = f;

	/* convert longitude */
	f = gps_pos_real2double(pos->longitude);
	if(pos->ew == 'W') {
		f *= -1;
	}
	*lon = f;

}


/**
 * Convert speed from nautical miles per hour (knots) to the more sane
 * meters per second.
 *
 * From Wikipedia:
 * 1 knot = 1.852 kilometres per hour (exactly)
 * 1 knot = 0.514 meters per second, 
 */
static gps_real_t gps_knots2mps(gps_real_t speed_knots)
{
	return (speed_knots * 257) / 500;
}


static int get_bigger_num_raised_to_ten(int n)
{
	int raised = 1;
	while(n) {
		n /= 10;
		raised *= 10;
	}
	return raised;
}

static int gps_parse_float(char **line, gps_real_t *f)
{
	char *tk;
	int n, fract;

	/* Latitude */
	tk = strnext(line, ".");
	if(!tk) {
		/* empty record */
		*f = 0;
		tk = strnext(line, gps_delim);
		if(!tk || strlen(tk)) {
			return -EINVAL;
		} else {
			return 0;
		}
	}
	n = atoi(tk);

	tk = strnext(line, gps_delim);
	if(!tk)
		return -EINVAL;
	fract = atoi(tk);

	if(fract > 10000) {
		return -EINVAL;
	}

	*f = gps_real_create(n, fract, get_bigger_num_raised_to_ten(fract));

	return 0;
}


static int gps_parse_position(char **line, struct gps_position *pos)
{
	char *tk;
	int stat;

	/* Latitude */
	stat = gps_parse_float(line, &pos->latitude);
	if(stat)
		return stat;

	/* N/S Indicator */
	tk = strnext(line, gps_delim);
	if(!tk)
		return -EINVAL;
	if(!strcmp(tk,"N")) 
		pos->ns = 'N';
	else if(!strcmp(tk,"S")) 
		pos->ns = 'S';
	else
		return -EINVAL;

	/* Longitude */
	stat = gps_parse_float(line, &pos->longitude);
	if(stat)
		return stat;

	/* E/W Indicator */
	tk = strnext(line, gps_delim);
	if(!tk)
		return -EINVAL;
	if(!strcmp(tk,"W")) 
		pos->ew = 'W';
	else if(!strcmp(tk,"E")) 
		pos->ew = 'E';
	else
		return -EINVAL;

	return 0;
}


static int gps_parse_utc_time(char **line, struct gps_utc_time *t)
{
	long n;
	char *tk;

	tk = strnext(line, ".");
	if(!tk)
		return -EINVAL;
	n = atol(tk);
	t->hour = (n/10000);
	t->min = (n/100) % 100;
	t->sec = n % 100;

	tk = strnext(line, gps_delim);
	if(!tk)
		return -EINVAL;
	t->ms = atoi(tk);

	return 0;
}


static int gps_parse_utc_date(char **line, struct gps_utc_date *d)
{
	long n;
	char *tk;

	tk = strnext(line, gps_delim);
	if(!tk)
		return -EINVAL;
	n = atol(tk);
	d->day = (n/10000);
	d->month = (n/100) % 100;
	d->year = (n) % 100 + 2000;
	
	return 0;
}


static int gps_parse_gpgga(char *line)
{
	struct gps_gpgga g;
	int stat;
	char *tk;

	/* UTC Time */
	stat = gps_parse_utc_time(&line, &g.utc_time);
	if(stat)
		return stat;

	/* Position */
	stat = gps_parse_position(&line, &g.pos);
	if(stat)
		return stat;

	/* Position Fix Indicator */
	tk = strnext(&line, gps_delim);
	if(!tk)
		return -EINVAL;
	g.pos_fix_ind = !!atoi(tk);

	/* Satelites Used */
	tk = strnext(&line, gps_delim);
	if(!tk)
		return -EINVAL;
	g.sats_used = atoi(tk);

	/* HDOP */
	stat = gps_parse_float(&line, &g.hdop);
	if(stat)
		return stat;

	/* MSL Altitude */
	stat = gps_parse_float(&line, &g.msl_altitude);
	if(stat)
		return stat;

	/* Units */
	tk = strnext(&line, gps_delim);
	if(!tk)
		return -EINVAL;
	if(strcmp(tk,"M")) 
		return -EINVAL;		/* we understand only meters */

	/* Geoid Separation */
	tk = strnext(&line, gps_delim);
	if(!tk)
		tk = "0";
	g.geoid_sep = atoi(tk);
	
	/* Units */
	tk = strnext(&line, gps_delim);
	if(!tk)
		return -EINVAL;
	if(strcmp(tk,"M")) 
		return -EINVAL;		/* we understand only meters */

	/* Ignore "Age of Differential Corrections" and "Diff Reference
	 * Corrections"
	 */

	/* Ok, publish */
	gpgga = g;
	gpgga_valid = true;

	return 0;
}


static int gps_parse_gpgsa(char *line)
{
	struct gps_gpgsa g;
	int i, stat;
	char *tk;

	/* Mode */
	tk = strnext(&line, gps_delim);
	if(!tk)
		return -EINVAL;
	if(!strcmp(tk,"M")) 
		g.mode = 'M';
	else if(!strcmp(tk,"A")) 
		g.mode = 'A';
	else
		return -EINVAL;

	/* Mode Fix */
	tk = strnext(&line, gps_delim);
	if(!tk)
		return -EINVAL;
	g.mode_fix = atoi(tk);

	/* Satellites Used */
	for(i=0; i<12; i++) {
		tk = strnext(&line, gps_delim);
		if(!tk)
			tk = "0";
		g.sats_used[i] = atoi(tk);
	}

	/* PDOP */
	stat = gps_parse_float(&line, &g.pdop);
	if(stat)
		return stat;

	/* HDOP */
	stat = gps_parse_float(&line, &g.hdop);
	if(stat)
		return stat;

	/* VDOP */
	stat = gps_parse_float(&line, &g.vdop);
	if(stat)
		return stat;

	/* Ok, publish */
	gpgsa = g;
	gpgsa_valid = true;

	return 0;
}


static int gps_parse_gprmc(char *line)
{
	struct gps_gprmc g;
	int stat;
	char *tk;

	/* UTC Time */
	stat = gps_parse_utc_time(&line, &g.utc_time);
	if(stat)
		return stat;

	/* Status */
	tk = strnext(&line, gps_delim);
	if(!tk)
		return -EINVAL;
	if(!strcmp(tk,"V")) 
		g.status = 'V';
	else if(!strcmp(tk,"A")) 
		g.status = 'A';
	else
		return -EINVAL;

	/* Position */
	stat = gps_parse_position(&line, &g.pos);
	if(stat)
		return stat;

	/* Speed Over Ground */
	stat = gps_parse_float(&line, &g.speed);
	if(stat)
		return stat;
	g.speed = gps_knots2mps(g.speed);
	
	/* Course Over Ground */
	stat = gps_parse_float(&line, &g.course);
	if(stat)
		return stat;

	/* Date */
	stat = gps_parse_utc_date(&line, &g.utc_date);
	if(stat)
		return stat;


	/* Ok, publish */
	gprmc = g;
	gprmc_valid = true;

	return 0;
}


static void parse_line(char *line)
{
	int n, nb1, nb2;
	char checksum = 0;
	char *tk, *p;
	int stat;

	assert(line);
	/* remove trailing whitespace */
	p = strchr(line, '\r');
	if(p)
		*p = '\0';
	p = strchr(line, '\n');
	if(p)
		*p = '\0';
	p = strchr(line, ' ');
	if(p)
		*p = '\0';

	/* check message format */
	if((line[0]!='$') || (strlen(line)<8) || (line[strlen(line)-3]!='*')) {
		/* invalid format */
		return;
	}

	/* check checksum */
	for(p=&line[1]; *p != '*'; p++) {
		assert(*p);
		checksum ^= *p;
	}
	assert(p[0] == '*');
	p[0] = '\0';	/* remove checksum from the line */
	assert(p[1] && p[2]);
	nb1 = hex2bin_nibble(p[1]);
	nb2 = hex2bin_nibble(p[2]);
	if((nb1 < 0) || (nb2 < 0))
		return;	/* invalid checksum number */
	n = (nb1<<4) | nb2;
	if(n != checksum) 
		return;	/* incorrect checksum */

	/* parse command */
	tk = strnext(&line, gps_delim);
	if(!tk)
		return;
	if(!strcmp(tk,"$GPGGA")) {
		stat = gps_parse_gpgga(line);
		if(stat) {
			gps_gpgga_error_counter++;
		}
	} else if(!strcmp(tk,"$GPGSA")) {
		stat = gps_parse_gpgsa(line);
		if(stat) {
			gps_gpgsa_error_counter++;
		}
	} else if(!strcmp(tk,"$GPRMC")) {
		stat = gps_parse_gprmc(line);
		if(stat) {
			gps_gprmc_error_counter++;
		}
	} else {
		gps_unrecognized_nmea_counter++;
	}
}

void gps_init(void)
{
	gps_hw_init();

	gpgga_valid = false;
}


void gps_poll(void)
{
	char *p;
	
	do {
		p = gps_hw_get_line();
		if(p) {
			parse_line(p);
			gps_hw_dealloc_line();
		}
	} while(p);
}

int gps_get_gpgga(struct gps_gpgga *g)
{
	if(gpgga_valid) {
		*g = gpgga;
		return 0;
	} else {
		return -EINVAL;
	}
}

int gps_get_gpgsa(struct gps_gpgsa *g)
{
	if(gpgsa_valid) {
		*g = gpgsa;
		return 0;
	} else {
		return -EINVAL;
	}
}

int gps_get_gprmc(struct gps_gprmc *g)
{
	if(gprmc_valid) {
		*g = gprmc;
		return 0;
	} else {
		return -EINVAL;
	}
}


void gps_print_polar(char buf[16], gps_real_t p)
{
	int n, deg, min, sec, sec_fract00;

	n = gps_real_int(p);
	deg = n/100;
	min = n%100;

	n = gps_real_fract10000(p);
	sec = (n * 60) / 10000;
	sec_fract00 = (((n*60) % 10000) * 100) / 10000;

	sprintf(buf, "%03d %02d'%02d.%02d\"", deg, min, sec, sec_fract00);
}

double gps_calc_distance(const struct gps_position *pos1, const struct gps_position *pos2)
{
	double lon1, lon2;
	double lat1, lat2;
	double res;

	gps_norm_pos(pos1, &lat1, &lon1);
	gps_norm_pos(pos2, &lat2, &lon2);

	/* Formula courtesy of http://williams.best.vwh.net/avform.htm */
	res = 2*asin(sqrt(pow((sin((lat1-lat2)/2)),2) + 
                 cos(lat1)*cos(lat2)*pow((sin((lon1-lon2)/2)),2)));

	return res;
}


float gps_calc_course(const struct gps_position *pos1, const struct gps_position *pos2)
{
	double lon1, lon2;
	double lat1, lat2;
	double res;

	gps_norm_pos(pos1, &lat1, &lon1);
	gps_norm_pos(pos2, &lat2, &lon2);

	/* Formulas courtesy of http://williams.best.vwh.net/avform.htm */
	if(cos(lat1) < FLT_MIN) {
		if(lat1 > 0)
			res = M_PI;
		else
			res = 2 * M_PI;
	} else {
#if 0
		res = fmod(atan2(sin(lon1-lon2)*cos(lat2), cos(lat1)*sin(lat2)-sin(lat1)*cos(lat2)*cos(lon1-lon2)), 2*M_PI);
#else
		double d;

		d = gps_calc_distance(pos1, pos2);
		if(sin(lon2-lon1) < 0) {
			res = acos((sin(lat2)-sin(lat1)*cos(d))/(sin(d)*cos(lat1)));
		} else {
			res = 2*M_PI-acos((sin(lat2)-sin(lat1)*cos(d))/(sin(d)*cos(lat1)));
		}
#endif
	}

	return res;
}


int gps_calc_course_degr(const struct gps_position *pos1, const struct gps_position *pos2)
{
#if 1
	return (int)(gps_calc_course(pos1, pos2) * 180 / M_PI);
#else
	static int i;
	if((i+=10) >= 360)
		i = 0;
	return i;
#endif
}


long gps_calc_distance_m(const struct gps_position *pos1, const struct gps_position *pos2)
{
	const double earth_radius = 6366.71 * 1000;

	return (long)(gps_calc_distance(pos1, pos2) * earth_radius);
}

