/****************************************************************************
 *
 * Project: -----
 *
 * Copyright by Olimex Ltd. All rights reserved.
 *
 * File: gps.h
 * Description: Hardware-agnostic GPS module
 * Developer: Dimitar Dimitrov ( dinuxbg,gmail.com )
 *
 * Last change: $Date: 2009-03-25 13:48:21 +0200 (Wed, 25 Mar 2009) $
 * Revision: $Revision: 29 $
 * Id: $Id: gps.h 658 2009-03-25 11:48:21Z dimitar $
 *
 ****************************************************************************/

#ifndef GPS_H
#define GPS_H

#include <stddef.h>
#include <stdint.h>
#include <stdbool.h>

#ifdef __cplusplus
extern "C" {
#endif


/** UTC time as reported by the GPS. */
struct gps_utc_time {
	int hour;
	int min;
	int sec;
	int ms;
};

/** UTC date as reported by the GPS. */
struct gps_utc_date {
	int day;
	int month;
	int year;
};


/* Use fixed point for real numbers */
typedef int32_t gps_real_t;


/** GPS position coordinates. */
struct gps_position {
	gps_real_t latitude;	/**< Degrees * 100 + minutes */
	char ns;		/**< North/South indicator */
	gps_real_t longitude;	/**< Degrees * 100 + minutes */
	char ew;		/**< East/West indicator */
};


/** Global Positioning System Fix Data. */
struct gps_gpgga {
	struct gps_utc_time utc_time;
	struct gps_position pos;
	bool pos_fix_ind;	/**< Position Fix Indicator:
				 * false: Fix not available or invalid;
				 * true: GPS SPS mode, Fix available.
				 */
	int sats_used;		/**< Number of satelites used to
				 * calculate fix.
				 */
	gps_real_t hdop;	/**< Horizontal Dilution of Precision. */
	gps_real_t msl_altitude;/**< Altitude above Mean Sea Level. */
	int geoid_sep;		/**< Separation from Geoid, can be blank. */
};


/** DOP and Active Satelites information. */
struct gps_gpgsa {
	char mode;		/**< M for Manual, A for Automatic. */
	int mode_fix;		/**< 1: Fix not available,
				 *   2: 2D position fix,
				 *   3: 3D position fix.
				 */
	int sats_used[12];	/**< SV on channels. */
	gps_real_t pdop;
	gps_real_t hdop;
	gps_real_t vdop;
};

/** Recommended minimum specific GNSS data. */
struct gps_gprmc {
	struct gps_utc_time utc_time;
	char status;		/**< A: Data Valid, V: Data Invalid. */
	struct gps_position pos;
	gps_real_t speed; 	/**< Speed over ground in m/s. */
	gps_real_t course; 	/**< Course over ground in degrees. */
	struct gps_utc_date utc_date;
};


/**
 * Get the integer (whole) part of the real number @a r.
 */
extern int gps_real_int(gps_real_t r);

/**
 * Get the fractional part f of the real number @a r, where
 * f = (r-gps_real_int(r)) * 10000
 */
extern unsigned int gps_real_fract10000(gps_real_t r);
extern unsigned int gps_real_fract100(gps_real_t r);
extern gps_real_t gps_real_create(int n, int fract, int fract_div);

extern void gps_init(void);
extern void gps_poll(void);
extern int gps_get_gpgga(struct gps_gpgga *g);
extern int gps_get_gpgsa(struct gps_gpgsa *g);
extern int gps_get_gprmc(struct gps_gprmc *g);


void gps_print_polar(char buf[16], gps_real_t p);

extern float gps_calc_course(const struct gps_position *pos1, const struct gps_position *pos2);
extern int gps_calc_course_degr(const struct gps_position *pos1, const struct gps_position *pos2);
extern long gps_calc_distance_m(const struct gps_position *pos1, const struct gps_position *pos2);


#ifdef __cplusplus
}
#endif

#endif	/* GPS_H */

