/****************************************************************************
 *
 * Project: -----
 *
 * Copyright by Olimex Ltd. All rights reserved.
 *
 * File: gsm-hw.c
 * Description: USART buffered receiver and transmitter for the GSM module
 * Developer: Dimitar Dimitrov ( dinuxbg,gmail.com )
 *
 * Last change: $Date: 2009-03-25 13:48:21 +0200 (Wed, 25 Mar 2009) $
 * Revision: $Revision: 29 $
 * Id: $Id: gsm-hw.c 658 2009-03-25 11:48:21Z dimitar $
 *
 ****************************************************************************/

#include "stm32f10x_lib.h"
#include "platform_config.h"

#include <assert.h>
#include <stddef.h>
#include <stdint.h>
#include <stdbool.h>

#include "gsm-hw.h"
#include "utils.h"


/**
 * @file
 * USART buffered receiver and transmitter for the GSM module
 *
 * Buffered receiver and transmitter of GSM messages.
 *
 */


/* DTR must be set low */
static void gsm_hw_init_dtr(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
  
	GPIO_ResetBits(GPIOA, GPIO_Pin_6);
}


/* PWRKEY must be open-drain, and must be asserted for 1.5 seconds. */
static void gsm_hw_init_pwrkey(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_OD;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
  
	GPIO_ResetBits(GPIOA, GPIO_Pin_4);
	counter_1ms = 0;
	while(counter_1ms < 1500);
	GPIO_SetBits(GPIOA, GPIO_Pin_4);
}

/* Initialize the auxiliary UEXT1 pins: PB8, PA5, PA6. */
static void gsm_hw_init_aux_uext1(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_OD;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_Init(GPIOB, &GPIO_InitStructure);
  
	GPIO_ResetBits(GPIOB, GPIO_Pin_6);

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5 | GPIO_Pin_6;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_OD;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
  
	GPIO_ResetBits(GPIOA, GPIO_Pin_5 | GPIO_Pin_6);

}



static void gsm_hw_gpio_init(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	/* Configure USART1 Tx (PA.09) as alternate function push-pull */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	/* Configure USART1 Rx (PA.10) as input floating */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
}

static void gsm_hw_enable_int(void)
{
	NVIC_InitTypeDef NVIC_InitStructure;

	/* Enable the USART1 Interrupt */
	NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQChannel;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	/* Enable USART1 Receive interrupts */
	USART_ITConfig(USART1, USART_IT_RXNE, ENABLE);
}


static void gsm_hw_init_usart(void)
{
	USART_InitTypeDef USART_InitStructure;
	USART_InitStructure.USART_BaudRate = 115200;
	USART_InitStructure.USART_WordLength = USART_WordLength_8b;
	USART_InitStructure.USART_StopBits = USART_StopBits_1;
	USART_InitStructure.USART_Parity = USART_Parity_No;
	USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
	/* Configure USART1 */
	USART_Init(USART1, &USART_InitStructure);

}


void gsm_hw_init(void)
{
	gsm_hw_init_dtr();
	gsm_hw_init_pwrkey();
	gsm_hw_init_aux_uext1();
	gsm_hw_gpio_init();
	gsm_hw_init_usart();
	gsm_hw_enable_int();

	/* Enable the USART1 */
	USART_Cmd(USART1, ENABLE);
}

/*-------------------------------------------------------------------------*/

#define LINE_SIZE	256
#define NUM_LINES	4
static char lines[NUM_LINES][LINE_SIZE];
static int current_line;
static int current_char_i;
static volatile int num_filled_lines;
  
/*******************************************************************************
* Function Name  : USART1_IRQHandler
* Description    : This function handles USART1 global interrupt request.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void USART1_IRQHandler(void)
{
	int c;

	if(USART_GetITStatus(USART1, USART_IT_RXNE) != RESET)
	{
		/* Read one byte from the receive data register */
		c = USART_ReceiveData(USART1); 

		if(c == '\n') {
			assert(current_char_i < LINE_SIZE);
			lines[current_line][current_char_i] = '\0';
			if(num_filled_lines < NUM_LINES) {
				num_filled_lines++;
				current_char_i = 0;
				current_line = (current_line+1) % NUM_LINES;
			}
		} else if((current_char_i < (LINE_SIZE-1)) && (c!='\r')) {
			lines[current_line][current_char_i] = c;
			current_char_i++;
		}
	}
}


char *gsm_hw_get_line(void)
{
	char *p;

	enter_critical_section();
	if(num_filled_lines>0) {
		unsigned int i;

		i = ((unsigned int)current_line - (unsigned int)num_filled_lines);
		i = i%NUM_LINES;
		
		p = &lines[i][0];
	} else {
		p = 0;
	}
	exit_critical_section();
	return p;
}


void gsm_hw_dealloc_line(void)
{
	enter_critical_section();
	assert(num_filled_lines>0);
	num_filled_lines--;
	exit_critical_section();
}


void gsm_hw_send_byte(int c)
{
	USART_SendData(USART1, c);

	/* Loop until USART1 DR register is empty */ 
	while(USART_GetFlagStatus(USART1, USART_FLAG_TXE) == RESET)
	{
	}
}

void gsm_hw_send_line(const char *line)
{
	while(*line) {
		/* Send one byte from USART1 to GSM */
		gsm_hw_send_byte(*line++);
	}
	gsm_hw_send_byte('\r');
	gsm_hw_send_byte('\n');
}


